#include "assembler.h"
#include <unistd.h>
#include <string.h>
#include <stdio.h>

static char msg[100];

void printmemref(memory_ref_t ref) {
  int flag = 0;

  switch(GET_MODE(ref.type)) {
  case CONSTANT:
    /*
    if (GET_TYPE(ref.type)==CODE_SECTION)
      break;
    */
    strcat(msg,"#");
    flag=1;
    break;
  case DIRECT:
    break;
  case INDIRECT:
    strcat(msg,"*");
    break;
  }
  /*
  if (!flag) {
    switch(GET_TYPE(ref.type)) {
    case DATA_SECTION:
      strcat(msg,"D");
      break;
    case CODE_SECTION:
      strcat(msg,"C");
      break;
    case STACK_SECTION:
      strcat(msg,"S");
      break;
    case BSS_SECTION:
      strcat(msg,"B");
      break;
    }
  }
  */
  sprintf(msg+strlen(msg),"%d",ref.addr);
}

void printlck(instruction_t ins) {
  printmemref(ins.datas.lck.arg1);
}

void printassign(instruction_t ins) {
  printmemref(ins.datas.assign.arg1);
  strcat(msg,",");
  printmemref(ins.datas.assign.arg2);
}

void printif(instruction_t ins) {
  printmemref(ins.datas.si.arg1);
  strcat(msg,",");
  printmemref(ins.datas.si.arg2);
  strcat(msg,",");
  printmemref(ins.datas.si.arg3);
}

void printjump(instruction_t ins) {
  printmemref(ins.datas.jump.arg1);
}

void printcall(instruction_t ins) {
  printmemref(ins.datas.call.arg1);
}

void printret(instruction_t ins) {
}

void printfork(instruction_t ins) {
  printmemref(ins.datas.fork.arg1);
}

void printexec(instruction_t ins) {
  printmemref(ins.datas.exec.arg1);
}

void printexit(instruction_t ins) {
  printmemref(ins.datas.exit.arg1);
}

void printsignal(instruction_t ins) {
  printmemref(ins.datas.signal.arg1);
  strcat(msg,",");
  printmemref(ins.datas.signal.arg2);
}

void printkill(instruction_t ins) {
  printmemref(ins.datas.kill.arg1);
  strcat(msg,",");
  printmemref(ins.datas.kill.arg2);
}

void printgetppid(instruction_t ins) {
  printmemref(ins.datas.getppid.arg1);
}

void printread(instruction_t ins) {
  printmemref(ins.datas.read.arg1);
}

void printwrite(instruction_t ins) {
  printmemref(ins.datas.write.arg1);
}

void printwait(instruction_t ins) {
  printmemref(ins.datas.wait.arg1);
}

int main() {
  int           i = 0;
  instruction_t ins;

  while ( read(0,&ins,sizeof(instruction_t))==sizeof(instruction_t) ) {
    if (ins.opcode>LAST_OPCODE)
      sprintf(msg,"C%-4d:UNK     ",i++);
    else
      sprintf(msg,"C%-4d:%-8s",i++,opcodes[ins.opcode]);
    switch(ins.opcode) {
    case ASSIGN_PLUS:
    case ASSIGN_MINUS:
    case ASSIGN_MULT:
    case ASSIGN:
      printassign(ins);
      break;
    case IF:
      printif(ins);
      break;
    case JUMP:
      printjump(ins);
      break;
    case CALL:
      printcall(ins);
      break;
    case RET:
      printret(ins);
      break;
    case FORK:
      printfork(ins);
      break;
    case EXEC:
      printexec(ins);
      break;
    case EXIT:
      printexit(ins);
      break;
    case SIGNAL:
      printsignal(ins);
      break;
    case WAIT:
      printwait(ins);
      break;
    case KILL:
      printkill(ins);
      break;
    case READ:
      printread(ins);
      break;
    case WRITE:
      printwrite(ins);
      break;
    case GETPPID:
      printgetppid(ins);
      break;
    case LCK:
    case UNLCK:
      printlck(ins);
      break;
    }
    printf("%s\n",msg);
  }
  return 0;
}
